//------------------------------------------------------------------------------
// AR-Calc  (Augmented Reality Calculator) ver 0.1  by E.Kako @kako.com (c)2012
//------------------------------------------------------------------------------

// Standard header
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// tesseract Lib header
#include "baseapi.h"
#include "strngs.h"

//OpenCV Lib header
#include "opencv2/opencv.hpp"

//OpenCV Libs
#pragma comment( lib, "lib/opencv_core242.lib")
#pragma comment( lib, "lib/opencv_highgui242.lib")
#pragma comment( lib, "lib/opencv_imgproc242.lib")

//------------------------------------------------------------------------------
// Main program
//------------------------------------------------------------------------------
int main()
{
	tesseract::TessBaseAPI tessBaseApi;
	
	tessBaseApi.Init(".","eng");
	tessBaseApi.SetVariable("tessedit_char_whitelist", "0123456789+-xX="); // limit char
	
	CvMemStorage *storage = 0;
	CvCapture *capture = 0;
	CvFont fnt;
	IplImage *frame, *frame_copy, *image;
	storage = cvCreateMemStorage(0);
	
	// open camera device
	capture = cvCaptureFromCAM(0);
	if (!capture) {
		fprintf(stderr, "Error: Camera not found.\n"); getchar();
		return -1;
	}
	
	// Open image window
	cvNamedWindow("image", 0);
	cvSetWindowProperty("image",CV_WND_PROP_FULLSCREEN,CV_WINDOW_FULLSCREEN); // fullscreen
	
	// frame
	frame      = cvQueryFrame(capture);
	frame_copy = cvCreateImage(cvGetSize(frame), 8, 3);
	image = cvCreateImage(cvGetSize(frame), 8, 1);
	
	// font init
	cvInitFont (&fnt, CV_FONT_HERSHEY_SIMPLEX,1,1,0,2,CV_AA);
	
	// main loop
	while(1) {
		if(cvWaitKey( 1 ) >= 0) { break; } // any key to quit
		
		frame = cvQueryFrame(capture); //capture a frame
		
		if (frame->origin == IPL_ORIGIN_TL) {
			cvCopy(frame, frame_copy); // copy frame
		} else {
			cvFlip(frame, frame_copy); // flip-copy frame
		}
		
		cvCvtColor( frame_copy, image, CV_RGB2GRAY); // convert to grayscale
		
		// OCR Tesseract
		{
			char *text = tessBaseApi.TesseractRect( 
				(unsigned char *)image->imageData, image->nChannels, image->widthStep,
				0, 0, image->width, image->height);
			{
				int len,pos,n,f,y;
				int i,j,k;
				int a,b,op,result;
				char c;
				char buf[256],t[256];
				
				len=strlen(text);
				pos=0;
				y=0;
				while (pos<len) {
					n=0;
					f=0;
					for (i=pos;i<len;i++) {
						c=text[i]; if ((c=='\r')||(c=='\n')) { break; }
						if (c=='=') { f=1; } // found (=)
						n++;
						if (i>254) { break; }
					}
					if (f==1) {
						j=0;
						k=0;
						for (i=0;i<n;i++) {
							c=text[pos+i];
							if ((i==0)&&(c=='+')) { continue; }
							if ((i==0)&&(c=='-')) { continue; }
							if ((i==0)&&(c=='x')) { continue; }
							if ((i==0)&&(c=='X')) { continue; }
							if ((i==0)&&(c=='=')) { continue; }
							if (c=='=') { break; }
							if (c=='+') { op=1; k=j; }
							if (c=='-') { op=2; k=j; }
							if (c=='x') { op=3; k=j; }
							if (c=='X') { op=3; k=j; }
							if (c==' ') { continue; }
							buf[j++]=c;
						}
						buf[j]='\0';
						if (k==0) {
							sscanf(buf,"%d",&result);
						} else {
							sscanf(buf,"%d",&a);
							sscanf(buf+k+1,"%d",&b);
							if (op==1) { result = a+b; }
							if (op==2) { result = a-b; }
							if (op==3) { result = a*b; }
						}
						sprintf(t,"%s = %d",buf,result);
						cvPutText(frame_copy, t , cvPoint(50, 50+y), &fnt, CV_RGB(255,0,0));
						y+=50;
					}
					pos += (n+1);
				}
			}
		}
		
		cvShowImage("image", frame_copy); // display
	}
	
	// release
	cvReleaseCapture(&capture);
	cvDestroyAllWindows();
	cvReleaseImage(&frame_copy);
	cvReleaseImage(&image);
	cvReleaseImage(&frame_copy);
	//cvReleaseImage(&frame);
		
	return 0;
}
